﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Collections.ObjectModel;
using Infragistics.Controls.Grids;

using TestStringIndexerSupport.Service1;

namespace TestStringIndexerSupport
{
    public partial class MainPage : UserControl
    {
        Service1Client client = new Service1Client();
        public MainPage()
        {
            InitializeComponent();

            client.GetDataCompleted += new EventHandler<GetDataCompletedEventArgs>(client_GetDataCompleted);
            client.GetDataAsync();


        }

        void client_GetDataCompleted(object sender, GetDataCompletedEventArgs e)
        {
            var collection = GetMixReports(e.Result);
            this.igGrid.ItemsSource = collection;

            foreach (var p in collection.First().Properties)
            {
                var newColumn = new TextColumn() { Key = "Properties[" + p.Name + "].Value", Width = new ColumnWidth(200, false), HeaderText = p.Name };
                igGrid.Columns.Add(newColumn);

                if (p.Value.GetType() == typeof(int))
                {
                    DataBarConditionalFormatRule myRule = new DataBarConditionalFormatRule();
                    myRule.DataBarDirection = DataBarDirection.Bidirectional;
                    myRule.DataBrush = new SolidColorBrush(Colors.Red);
                    myRule.NegativeDataBrush = new SolidColorBrush(Colors.Yellow);
                    myRule.StyleScope = StyleScope.Cell;
                    myRule.UseNegativeDataBar = true;

                    newColumn.ConditionalFormatCollection.Add(myRule);

                    //you need to clear all operators after the column is added to the grid
                    //because the default operators wont work since the bound type is Object
                    newColumn.FilterColumnSettings.RowFilterOperands.Clear();

                    //Here you could add your custom Filtering operators like the GreaterThan. 
                    newColumn.FilterColumnSettings.RowFilterOperands.Add(new GreaterThan() { ColumnKey = p.Name });
                }

            }
        }

        private ObservableCollection<File> GetMixReports(IEnumerable<Dictionary<string, object>> reportCol)
        {
            //ObservableCollection<File> files1 = e.Result;
            ObservableCollection<File> mixReportCol = new ObservableCollection<File>();

            foreach (var rptDic in reportCol)
            {
                NameValuePairCollection nvpc = new NameValuePairCollection();
                foreach (var item in rptDic)
                {
                    nvpc.Add(new NameValuePair { Name = item.Key, Value = item.Value });
                }
                mixReportCol.Add(new File { Properties = nvpc });
            }

            //for (int j = 0; j < files1.Count; j++)
            //{
            //    NameValPairCol nvpc = new NameValPairCol();

            //    foreach (var item in files1[j].Properties)
            //    {
            //        nvpc.Add(new NameValPair { Name = item.Name, Value = item.Value });
            //    }

            //    files2.Add(new MixReport { Properties = nvpc });
            //}
            return mixReportCol;

        }
    }

    public class NameValuePair
    {
        public string Name { get; set; }
        public object Value { get; set; }
    }

    public class NameValuePairCollection : System.Collections.ObjectModel.KeyedCollection<string, NameValuePair>
    {
        protected override string GetKeyForItem(NameValuePair item)
        {
            return item.Name;
        }
    }

    public class File
    {
        public NameValuePairCollection Properties { get; set; }
    }


    public class GreaterThan : FilterOperand
    {
        public string ColumnKey { get; set; }

        protected override string DefaultDisplayName
        {
            get
            {
                return "Greater Than";
            }
        }
        public override bool RequiresFilteringInput
        {
            get
            {
                return true;
            }
        }


        public override System.Linq.Expressions.Expression FilteringExpression(object value)
        {
            int intValue;

            //put some general purpose logic here which will used if the provided value cannot be parsed to int
            System.Linq.Expressions.Expression<Func<File, bool>> expr = item => true;

            if (Int32.TryParse((string)value, out intValue))
            {
                expr = item => item.Properties.Any(kvp => kvp.Name == this.ColumnKey) && (int)item.Properties[this.ColumnKey].Value > intValue;
            }
            return expr;
        }

        public int Convert(object o)
        {
            int a = (int)o;
            return a;
        }
    }

}
